@@ -1,7 +1,7 @@
 /*
-* libslack - http://libslack.org/
+* libslack - https://libslack.org
 *
-* Copyright (C) 1999-2002, 2004, 2010, 2020 raf <raf@raf.org>
+* Copyright (C) 1999-2002, 2004, 2010, 2020-2023 raf <raf@raf.org>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
@@ -16,7 +16,7 @@
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, see <https://www.gnu.org/licenses/>.
 *
-* 20201111 raf <raf@raf.org>
+* 20230819 raf <raf@raf.org>
 */
 
 /*
@@ -72,8 +72,8 @@ flexible behaviour. The message types catered for are: normal, verbose,
 debug, error, fatal error, dump and alert messages. All messages are created
 and sent with I<printf(3)>-like syntax. The destinations for these messages
 are configurable by the client. Calling I<prog_init(3)> causes normal and
-verbose messages to be sent to standard output; debug, error, fatal error,
-dump and alert messages to be sent to standard error.
+verbose messages to be sent to standard output; and debug, error, fatal
+error, dump and alert messages to be sent to standard error.
 
 Calls to I<prog_set_out(3)>, I<prog_out_fd(3)>, I<prog_out_stdout(3)>,
 I<prog_out_file(3)>, I<prog_out_syslog(3)> and I<prog_out_none(3)> cause
@@ -132,14 +132,14 @@ details.
 =item C<void msg(const char *format, ...)>
 
 Outputs a message to the program's normal message destination. C<format> is
-a I<printf(3)>-like format string and processes any remaining arguments in
+a I<printf(3)>-like format string which processes any remaining arguments in
 the same way as I<printf(3)>.
 
 B<Warning: Do not under any circumstances ever pass a non-literal string as
 the format argument unless you know exactly how many conversions will take
 place. Being careless with this is a very good way to build potential
-security holes into your programs. The same is true for all functions that
-take a printf()-like format string as an argument.>
+security vulnerabilities into your programs. The same is true for all
+functions that take a printf()-like format string as an argument.>
 
     msg(buf);       // EVIL
     msg("%s", buf); // GOOD
@@ -179,10 +179,10 @@ void vmsg(const char *format, va_list args)
 Outputs a verbose message to the program's normal message destination if
 C<level> is less than or equal to the program's current verbosity level. If
 the program's name has been supplied using I<prog_set_name(3)>, the message
-will be preceded by the name, a colon and a space. The message is also
+will be preceded by the name, a colon, and a space. The message is also
 preceded by as many spaces as the message level. This indents messages
 according to their verbosity. C<format> is a I<printf(3)>-like format string
-and processes any remaining arguments in the same way as I<printf(3)>. The
+which processes any remaining arguments in the same way as I<printf(3)>. The
 message is followed by a newline.
 
 =cut
@@ -234,16 +234,19 @@ C<level> satisfies the program's current debug level. The debug level is
 broken into two components. The low byte specifies the level. The next three
 bytes specify a section within which the level applies. Debug messages with
 a section value whose bits overlap those of the program's current debug
-section and with a level that is less than or equal to the program's current
-debug level are emitted. As a convenience, if the program's current debug
-section is zero, debug messages with a sufficiently small level are emitted
-regardless of the message section. See I<prog_set_debug_level(3)> for
-examples. If the program's name has been supplied using I<prog_set_name(3)>,
-the message will be preceded by the name, a colon and a space. The message
-is also preceded by as many spaces as the debug level. This indents debug
-messages according to their debug level. C<format> is a I<printf(3)>-like
-format string and processes any remaining arguments in the same way as
-I<printf(3)>. The message is followed by a newline.
+section, and with a level that is less than or equal to the program's
+current debug level, are emitted. As a convenience, if the program's current
+debug section is zero, debug messages with a sufficiently small level are
+emitted, regardless of the message section. See I<prog_set_debug_level(3)>
+for examples. If the program's name has been supplied using
+I<prog_set_name(3)>, the message will be preceded by the name, a colon, and
+a space. This is followed by the string C<"debug:">. If C<level> specifies a
+non-zero section, it is included in the debug message, after a space and
+surrounded by square brackets. This is followed by as many spaces as the
+debug level. This indents debug messages according to their debug level.
+C<format> is a I<printf(3)>-like format string which processes any remaining
+arguments in the same way as I<printf(3)>. The message is followed by a
+newline.
 
 =cut
 
@@ -307,9 +310,10 @@ void vdebugf(size_t level, const char *format, va_list args)
 
 Outputs an error message to the program's error message destination. If the
 program's name has been supplied using I<prog_set_name(3)>, the message will
-be preceded by the name, a colon and a space. C<format> is a
-I<printf(3)>-like format string and processes any remaining arguments in the
-same way as I<printf(3)>. The message is followed by a newline. Returns -1.
+be preceded by the name, a colon, and a space. C<format> is a
+I<printf(3)>-like format string which processes any remaining arguments in
+the same way as I<printf(3)>. The message is followed by a newline. Returns
+-1.
 
 =cut
 
@@ -353,14 +357,14 @@ int verror(const char *format, va_list args)
 
 =item C<void fatal(const char *format, ...)>
 
-Outputs an error message to the program's error message destination and then
-calls I<exit(3)> with a return code of C<EXIT_FAILURE>. If the program's
-name was supplied using I<prog_set_name(3)>, the message will be preceded
-by the name, a colon and a space. This is followed by the string C<"fatal: ">.
-C<format> is a I<printf(3)>-like format string and processes any remaining
-arguments in the same way as I<printf(3)>. The message is followed by a
-newline. B<Note:> Never use this in a library. Only an application can
-decide which errors are fatal.
+Outputs an error message to the program's error message destination, and
+then calls I<exit(3)> with a return code of C<EXIT_FAILURE>. If the
+program's name was supplied using I<prog_set_name(3)>, the message will be
+preceded by the name, a colon, and a space. This is followed by the string
+C<"fatal: ">. C<format> is a I<printf(3)>-like format string which processes
+any remaining arguments in the same way as I<printf(3)>. The message is
+followed by a newline. B<Note:> Never use this in a library. Only an
+application can decide which errors are fatal.
 
 =cut
 
@@ -399,10 +403,10 @@ void vfatal(const char *format, va_list args)
 
 Outputs an error message to the program's error message destination and then
 calls I<abort(3)>. If the program's name was supplied using
-I<prog_set_name(3)>, the message will be preceded by the name, a colon and
+I<prog_set_name(3)>, the message will be preceded by the name, a colon, and
 a space. This is followed by the string C<"dump: ">. C<format> is a
-I<printf(3)>-like format string and processes any remaining arguments in the
-same way as I<printf(3)>. The message is followed by a newline. B<Note:>
+I<printf(3)>-like format string which processes any remaining arguments in
+the same way as I<printf(3)>. The message is followed by a newline. B<Note:>
 Never use this in a library. Only an application can decide which errors are
 fatal.
 
@@ -443,8 +447,8 @@ void vdump(const char *format, va_list args)
 
 Outputs an alert message of the given C<priority> to the program's alert
 message destination. If the program's name has been supplied using
-I<prog_set_name(3)>, the message will be preceded by the name, a colon and
-a space. C<format> is a I<printf(3)>-like format string and processes any
+I<prog_set_name(3)>, the message will be preceded by the name, a colon, and
+a space. C<format> is a I<printf(3)>-like format string which processes any
 remaining arguments in the same way as I<printf(3)>. The message is followed
 by a newline. Note that this only works when the program's alert message
 destination is a simple syslog destination. If the alert message destination
@@ -506,8 +510,8 @@ void valert(int priority, const char *format, va_list args)
 =item C<void debugsysf(size_t level, const char *format, ...)>
 
 Equivalent to I<debugf(3)> except that the message is followed by a colon, a
-space, the string representation of C<errno> and a newline (rather than just
-a newline).
+space, the string representation of C<errno>, and a newline (rather than
+just a newline).
 
 =cut
 
@@ -551,8 +555,8 @@ void vdebugsysf(size_t level, const char *format, va_list args)
 =item C<int errorsys(const char *format, ...)>
 
 Equivalent to I<error(3)> except that the message is followed by a colon, a
-space, the string representation of C<errno> and a newline (rather than just
-a newline).
+space, the string representation of C<errno>, and a newline (rather than
+just a newline).
 
 =cut
 
@@ -591,8 +595,8 @@ int verrorsys(const char *format, va_list args)
 =item C<void fatalsys(const char *format, ...)>
 
 Equivalent to I<fatal(3)> except that the message is followed by a colon, a
-space, the string representation of C<errno> and a newline (rather than just
-a newline).
+space, the string representation of C<errno>, and a newline (rather than
+just a newline).
 
 =cut
 
@@ -630,8 +634,8 @@ void vfatalsys(const char *format, va_list args)
 =item C<void dumpsys(const char *format, ...)>
 
 Equivalent to I<dump(3)> except that the message is followed by a colon, a
-space, the string representation of C<errno> and a newline (rather than just
-a newline).
+space, the string representation of C<errno>, and a newline (rather than
+just a newline).
 
 =cut
 
@@ -669,8 +673,8 @@ void vdumpsys(const char *format, va_list args)
 =item C<void alertsys(int priority, const char *format, ...)>
 
 Equivalent to I<alert(3)> except that the message is followed by a colon, a
-space, the string representation of C<errno> and a newline (rather than just
-a newline).
+space, the string representation of C<errno>, and a newline (rather than
+just a newline).
 
 =cut
 
@@ -740,7 +744,7 @@ void *(set_errnull)(int errnum)
 =item C<void (*set_errnullf(int errnum))()>
 
 Sets C<errno> to C<errnum> and returns C<null> as a function pointer.
-This is useful because ISO C doesn't like casting normal pointers
+This is useful because I<ISO C> doesn't like casting normal pointers
 into function pointers.
 
 =cut
@@ -762,6 +766,10 @@ with extra parentheses. e.g.
 
     debug((1, "rc=%d", rc))
 
+Note that the I<debug()> macro always generates its own semicolon. This used
+to only be the case when C<NDEBUG> was not defined, but it is now
+consistent.
+
 =item C< #define vdebug(args)>
 
 Calls I<vdebugf(3)> unless C<NDEBUG> is defined. C<args> must be supplied
@@ -769,6 +777,10 @@ with extra parentheses. e.g.
 
     vdebug((1, format, args))
 
+Note that the I<vdebug()> macro always generates its own semicolon. This
+used to only be the case when C<NDEBUG> was not defined, but it is now
+consistent.
+
 =item C< #define debugsys(args)>
 
 Calls I<debugsysf(3)> unless C<NDEBUG> is defined. C<args> must be supplied
@@ -776,6 +788,10 @@ with extra parentheses. e.g.
 
     debugsys((1, "fd=%d", fd))
 
+Note that the I<debugsys()> macro always generates its own semicolon. This
+used to only be the case when C<NDEBUG> was not defined, but it is now
+consistent.
+
 =item C< #define vdebugsys(args)>
 
 Calls I<vdebugsysf(3)> unless C<NDEBUG> is defined. C<args> must be supplied
@@ -783,6 +799,10 @@ with extra parentheses. e.g.
 
     vdebugsys((1, format, args))
 
+Note that the I<vdebugsys()> macro always generates its own semicolon. This
+used to only be the case when C<NDEBUG> was not defined, but it is now
+consistent.
+
 =item C< #define check(cond, mesg)>
 
 Like I<assert(3)> but includes a string argument, C<mesg>, for including an
@@ -790,7 +810,7 @@ explanation of the condition tested and then calls I<dump(3)> to terminate
 the program. This means the message will be sent to the right place(s)
 rather than just to C<stderr>. B<Note:> Like I<assert(3)>, this function is
 largely useless. It should never be left in production code (because it's
-rude) so you need to write code to handle error conditions properly anyway.
+rude), so you need to write code to handle error conditions properly anyway.
 You might as well not bother using I<assert(3)> or I<check(3)> in the first
 place.
 
@@ -798,7 +818,7 @@ place.
 
 =head1 MT-Level
 
-MT-Safe
+I<MT-Safe>
 
 =head1 EXAMPLES
 
@@ -837,7 +857,7 @@ I<printf(3)>
 
 =head1 AUTHOR
 
-20201111 raf <raf@raf.org>
+20230819 raf <raf@raf.org>
 
 =cut
 
@@ -852,6 +872,7 @@ I<printf(3)>
 #include <sys/wait.h>
 #include <sys/types.h>
 #include <sys/stat.h>
+#include <sys/resource.h>
 
 #include "str.h"
 
@@ -1008,9 +1029,9 @@ int main(int ac, char **av)
 			}
 
 			if (!WIFEXITED(status) || WEXITSTATUS(status) != EXIT_FAILURE)
-				++errors, printf("Test7: failed: %s %d\n",
+				++errors, printf("Test7: failed: %s %d (expected: exit code %d)\n",
 					WIFSIGNALED(status) ? "received signal" : "exit code",
-					WIFSIGNALED(status) ? WTERMSIG(status) : WEXITSTATUS(status));
+					WIFSIGNALED(status) ? WTERMSIG(status) : WEXITSTATUS(status), EXIT_FAILURE);
 		}
 	}
 
@@ -1037,7 +1058,6 @@ int main(int ac, char **av)
 
 		default:
 		{
-			struct stat statbuf[1];
 			int status;
 
 			if (waitpid(pid, &status, 0) == -1)
@@ -1047,18 +1067,28 @@ int main(int ac, char **av)
 			}
 
 			if (!WIFSIGNALED(status) || WTERMSIG(status) != SIGABRT)
-				++errors, printf("Test8: failed: %s %d\n",
-					WIFSIGNALED(status) ? "received signal" : "exit code",
-					WIFSIGNALED(status) ? WTERMSIG(status) : WEXITSTATUS(status));
-
-			if (stat(core, statbuf) == -1 && errno == ENOENT &&
-				stat(core2, statbuf) == -1 && errno == ENOENT)
-				++errors, printf("Test8: failed: no core file produced (ulimit?)\n");
-			else
 			{
-				unlink(core);
-				unlink(core2);
+				++errors, printf("Test8: failed: %s %d (expected: received signal %d)\n",
+					WIFSIGNALED(status) ? "received signal" : "exit code",
+					WIFSIGNALED(status) ? WTERMSIG(status) : WEXITSTATUS(status), SIGABRT);
 			}
+
+			/* Check for core dump, but ulimit might prevent them */
+
+#ifdef WCOREDUMP
+			if (WIFSIGNALED(status) && !WCOREDUMP(status))
+			{
+				struct rlimit limit[1] = {{ 0, 0 }};
+
+				if (getrlimit(RLIMIT_CORE, limit) != -1 && limit->rlim_cur != 0)
+					printf("Warning: dump() produced no core file (even though ulimit -c %d)\n", (int)limit->rlim_cur);
+			}
+#endif
+
+			/* Core files might be anywhere these days, but they might be here */
+
+			unlink(core);
+			unlink(core2);
 		}
 	}
 
@@ -1106,9 +1136,9 @@ int main(int ac, char **av)
 			}
 
 			if (!WIFEXITED(status) || WEXITSTATUS(status) != EXIT_FAILURE)
-				++errors, printf("Test11: failed: %s %d\n",
+				++errors, printf("Test11: failed: %s %d (expected: exit code %d)\n",
 					WIFSIGNALED(status) ? "received signal" : "exit code",
-					WIFSIGNALED(status) ? WTERMSIG(status) : WEXITSTATUS(status));
+					WIFSIGNALED(status) ? WTERMSIG(status) : WEXITSTATUS(status), EXIT_FAILURE);
 		}
 	}
 
@@ -1136,7 +1166,6 @@ int main(int ac, char **av)
 
 		default:
 		{
-			struct stat statbuf[1];
 			int status;
 
 			if (waitpid(pid, &status, 0) == -1)
@@ -1147,18 +1176,26 @@ int main(int ac, char **av)
 			}
 
 			if (!WIFSIGNALED(status) || WTERMSIG(status) != SIGABRT)
-				++errors, printf("Test12: failed: %s %d\n",
+				++errors, printf("Test12: failed: %s %d (expected: received signal %d)\n",
 					WIFSIGNALED(status) ? "received signal" : "exit code",
-					WIFSIGNALED(status) ? WTERMSIG(status) : WEXITSTATUS(status));
+					WIFSIGNALED(status) ? WTERMSIG(status) : WEXITSTATUS(status), SIGABRT);
 
-			if (stat(core, statbuf) == -1 && errno == ENOENT &&
-				stat(core2, statbuf) == -1 && errno == ENOENT)
-				++errors, printf("Test12: failed: no core file produced (ulimit?)\n");
-			else
+			/* Check for core dump, but ulimit might prevent them */
+
+#ifdef WCOREDUMP
+			if (WIFSIGNALED(status) && !WCOREDUMP(status))
 			{
-				unlink(core);
-				unlink(core2);
+				struct rlimit limit[1] = {{ 0, 0 }};
+
+				if (getrlimit(RLIMIT_CORE, limit) != -1 && limit->rlim_cur != 0)
+					printf("Warning: dump() produced no core file (even though ulimit -c %d)\n", (int)limit->rlim_cur);
 			}
+#endif
+
+			/* Core files might be anywhere these days, but they might be here */
+
+			unlink(core);
+			unlink(core2);
 		}
 	}
 
@@ -1198,9 +1235,9 @@ int main(int ac, char **av)
 			}
 
 			if (!WIFEXITED(status) || WEXITSTATUS(status) != EXIT_SUCCESS)
-				++errors, printf("Test13: failed: %s %d\n",
+				++errors, printf("Test13: failed: %s %d (expected: exit code %d)\n",
 					WIFSIGNALED(status) ? "received signal" : "exit code",
-					WIFSIGNALED(status) ? WTERMSIG(status) : WEXITSTATUS(status));
+					WIFSIGNALED(status) ? WTERMSIG(status) : WEXITSTATUS(status), EXIT_SUCCESS);
 		}
 	}
 
@@ -1228,7 +1265,6 @@ int main(int ac, char **av)
 
 		default:
 		{
-			struct stat statbuf[1];
 			int status;
 
 			if (waitpid(pid, &status, 0) == -1)
@@ -1239,18 +1275,26 @@ int main(int ac, char **av)
 			}
 
 			if (!WIFSIGNALED(status) || WTERMSIG(status) != SIGABRT)
-				++errors, printf("Test14: failed: %s %d\n",
+				++errors, printf("Test14: failed: %s %d (expected: received signal %d)\n",
 					WIFSIGNALED(status) ? "received signal" : "exit code",
-					WIFSIGNALED(status) ? WTERMSIG(status) : WEXITSTATUS(status));
+					WIFSIGNALED(status) ? WTERMSIG(status) : WEXITSTATUS(status), SIGABRT);
 
-			if (stat(core, statbuf) == -1 && errno == ENOENT &&
-				stat(core2, statbuf) == -1 && errno == ENOENT)
-				++errors, printf("Test14: failed: no core file produced (ulimit?)\n");
-			else
+			/* Check for core dump, but ulimit might prevent them */
+
+#ifdef WCOREDUMP
+			if (WIFSIGNALED(status) && !WCOREDUMP(status))
 			{
-				unlink(core);
-				unlink(core2);
+				struct rlimit limit[1] = {{ 0, 0 }};
+
+				if (getrlimit(RLIMIT_CORE, limit) != -1 && limit->rlim_cur != 0)
+					printf("Warning: dump() produced no core file (even though ulimit -c %d)\n", (int)limit->rlim_cur);
 			}
+#endif
+
+			/* Core files might be anywhere these days, but they might be here */
+
+			unlink(core);
+			unlink(core2);
 		}
 	}
 
